
.icon-play:before { content: '\e800'; } /* '' */
.icon-pause:before { content: '\e801'; } /* '' */
.icon-to-end:before { content: '\e802'; } /* '' */
.icon-home:before { content: '\e802'; } /* '' */
.icon-to-start:before { content: '\e803'; } /* '' */
.icon-volume-down:before { content: '\e804'; } /* '' */
.icon-search:before { content: '\e804'; } /* '' */
.icon-volume-up:before { content: '\e805'; } /* '' */
.icon-folder-open:before { content: '\e805'; } /* '' */
.icon-mic-1:before { content: '\e806'; } /* '' */
.icon-info-circled:before { content: '\e806'; } /* '' */
.icon-mic-circled:before { content: '\e807'; } /* '' */
.icon-cog:before { content: '\e807'; } /* '' */
.icon-cancel-circled-outline:before { content: '\e808'; } /* '' */
.icon-cog-alt:before { content: '\e808'; } /* '' */
.icon-cancel-circled-1:before { content: '\e809'; } /* '' */
.icon-trash-empty:before { content: '\e80a'; } /* '' */
.icon-edit-1:before { content: '\e80b'; } /* '' */
.icon-pencil:before { content: '\e80c'; } /* '' */
.icon-down-big:before { content: '\e80c'; } /* '' */
.icon-th-list:before { content: '\e80d'; } /* '' */
.icon-left-big:before { content: '\e80d'; } /* '' */
.icon-th-list-outline:before { content: '\e80e'; } /* '' */
.icon-right-big:before { content: '\e80e'; } /* '' */
.icon-music:before { content: '\e80f'; } /* '' */
.icon-list-add:before { content: '\e810'; } /* '' */
.icon-up-big:before { content: '\e810'; } /* '' */
.icon-trash:before { content: '\e811'; } /* '' */
.icon-plus:before { content: '\e812'; } /* '' */
.icon-plus-circled:before { content: '\e813'; } /* '' */
.icon-home-1:before { content: '\e814'; } /* '' */
.icon-home-2:before { content: '\e815'; } /* '' */
.icon-search-1:before { content: '\e816'; } /* '' */
.icon-search-3:before { content: '\e817'; } /* '' */
.icon-info-circled-1:before { content: '\e818'; } /* '' */
.icon-info:before { content: '\e819'; } /* '' */
.icon-left-circled:before { content: '\f0a8'; } /* '' */
.icon-right-circled:before { content: '\f0a9'; } /* '' */
.icon-up-circled:before { content: '\f0aa'; } /* '' */
.icon-down-circled:before { content: '\f0ab'; } /* '' */
.icon-list-bullet:before { content: '\f0ca'; } /* '' */
.icon-list-numbered:before { content: '\f0cb'; } /* '' */
.icon-sort-down:before { content: '\f0dd'; } /* '' */
.icon-sort-up:before { content: '\f0de'; } /* '' */
.icon-folder-open-empty:before { content: '\f115'; } /* '' */
.icon-search-2:before { content: '\f50d'; } /* '' */
