#score{
	position: absolute;
	background-image: url("/static/images/stage.jpg");
	background-size: cover;
	left: 0px;
	top: 0px;
	z-index: 10;
	height: 100vh;
	width: 100vw;
}

#your-score-text{
	font-size: 5rem;
	width: 100vw;
	text-align: center;
	position: absolute;
	top: 15vh;
	text-shadow: -1px 1px 5px black;
}

#score-number-text{
	position: absolute;
	width: 100vw;
	text-align: center;
	font-size: 12rem;
	top: 25vh;
	font-weight: 800;
	text-shadow: -2px 2px 6px black;
}

#score-review-text{
	position: absolute;
	width: 100vw;
	text-align: center;
	font-size: 3rem;
	top: 62vh;
	text-shadow: -1px 1px 5px black;
}

canvas {
	position: absolute;
	top: 0;
	left: 0;
	width: 100%;
	height: 100%;
	/* z-index: 1; */
 }
