body {
  margin: 0;
  box-sizing: border-box;
  overflow: hidden;
}

#screensaver {
  position: absolute;
  left: 0px;
  top: 0px;
  z-index: 10;
  height: 100vh;
  width: 100vw;
  background-color: #111;
}

#dvd {
  position: absolute;
  left: 0px;
  top: 0px;
  height: 200px;
  width: 420px;
  z-index: 11;
  border-radius: 10px;
  display: flex;
  justify-content: right;
  align-items: flex-end;
  padding: 10px;
  background-image: url("/logo");
  background-repeat: no-repeat;
  background-size: 300px;
  background-position: contain;
}
